@@ -23,30 +23,32 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #ifndef KWIN_SLIDE_H
 #define KWIN_SLIDE_H
 
-// KDE
+// kwineffects
 #include <kwineffects.h>
 
-// Qt
-#include <QObject>
-#include <QTimeLine>
-
 namespace KWin
 {
 
 class SlideEffect : public Effect
 {
     Q_OBJECT
+    Q_PROPERTY(int duration READ duration)
+    Q_PROPERTY(int horizontalGap READ horizontalGap)
+    Q_PROPERTY(int verticalGap READ verticalGap)
+    Q_PROPERTY(bool slideDocks READ slideDocks)
+    Q_PROPERTY(bool slideBackground READ slideBackground)
+
 public:
     SlideEffect();
 
     void reconfigure(ReconfigureFlags) override;
 
-    void prePaintScreen(ScreenPrePaintData& data, int time) override;
-    void paintScreen(int mask, QRegion region, ScreenPaintData& data) override;
+    void prePaintScreen(ScreenPrePaintData &data, int time) override;
+    void paintScreen(int mask, QRegion region, ScreenPaintData &data) override;
     void postPaintScreen() override;
 
-    void prePaintWindow(EffectWindow* w, WindowPrePaintData& data, int time) override;
-    void paintWindow(EffectWindow* w, int mask, QRegion region, WindowPaintData& data) override;
+    void prePaintWindow(EffectWindow *w, WindowPrePaintData &data, int time) override;
+    void paintWindow(EffectWindow *w, int mask, QRegion region, WindowPaintData &data) override;
 
     bool isActive() const override {
         return m_active;
@@ -58,10 +60,16 @@ public:
 
     static bool supported();
 
+    int duration() const;
+    int horizontalGap() const;
+    int verticalGap() const;
+    bool slideDocks() const;
+    bool slideBackground() const;
+
 private Q_SLOTS:
-    void desktopChanged(int old, int current, EffectWindow* with);
-    void windowAdded(EffectWindow* w);
-    void windowDeleted(EffectWindow* w);
+    void desktopChanged(int old, int current, EffectWindow *with);
+    void windowAdded(EffectWindow *w);
+    void windowDeleted(EffectWindow *w);
 
     void numberDesktopsChanged(uint old);
     void numberScreensChanged();
@@ -72,29 +80,31 @@ private:
     int workspaceWidth() const;
     int workspaceHeight() const;
 
-    bool isTranslated(const EffectWindow* w) const;
-    bool isPainted(const EffectWindow* w) const;
+    bool isTranslated(const EffectWindow *w) const;
+    bool isPainted(const EffectWindow *w) const;
 
-    bool shouldForceBlur(const EffectWindow* w) const;
-    bool shouldForceBackgroundContrast(const EffectWindow* w) const;
-    bool shouldElevate(const EffectWindow* w) const;
+    bool shouldForceBlur(const EffectWindow *w) const;
+    bool shouldForceBackgroundContrast(const EffectWindow *w) const;
+    bool shouldElevate(const EffectWindow *w) const;
 
-    void start(int old, int current, EffectWindow* movingWindow = nullptr);
+    void start(int old, int current, EffectWindow *movingWindow = nullptr);
     void stop();
 
 private:
     int m_hGap;
     int m_vGap;
     bool m_slideDocks;
+    bool m_slideBackground;
 
     bool m_active = false;
-    QTimeLine m_timeline;
+    TimeLine m_timeLine;
     QPoint m_startPos;
     QPoint m_diff;
-    EffectWindow* m_movingWindow = nullptr;
+    EffectWindow *m_movingWindow = nullptr;
 
     struct {
         int desktop;
+        bool firstPass;
         bool lastPass;
         QPoint translation;
 
@@ -102,13 +112,38 @@ private:
     } m_paintCtx;
 
     struct {
-        QList<EffectWindow*> blur;
-        QList<EffectWindow*> backgroundContrast;
+        EffectWindowList blur;
+        EffectWindowList backgroundContrast;
     } m_forcedRoles;
 
-    QList<EffectWindow*> m_elevatedWindows;
+    EffectWindowList m_elevatedWindows;
 };
 
-} // namespace
+inline int SlideEffect::duration() const
+{
+    return m_timeLine.duration().count();
+}
+
+inline int SlideEffect::horizontalGap() const
+{
+    return m_hGap;
+}
+
+inline int SlideEffect::verticalGap() const
+{
+    return m_vGap;
+}
+
+inline bool SlideEffect::slideDocks() const
+{
+    return m_slideDocks;
+}
+
+inline bool SlideEffect::slideBackground() const
+{
+    return m_slideBackground;
+}
+
+} // namespace KWin
 
 #endif
